import re
from datetime import datetime
from commodutil import pandasutil as pdu
import pandas as pd
import urllib3
from commodplot import jinjautils as ju
from commodplot.messaging import compose_and_send_jinja_report
from excel_scraper import excel_scraper

from oilanalytics.utils import chartutils as cu

fileloc = "https://ir.eia.gov/wpsr/psw09.xls"

canada_importa_file_loc = (
    "https://www.eia.gov/dnav/pet/hist_xls/W_EPC0_IM0_NUS-NCA_MBBLDw.xls"
)

eia_url = "https://www.eia.gov/opendata/qb.php?sdid=PET.%s.W"

sheets_to_parse = [
    "Contents",
    "Data 1",
    "Data 2",
    "Data 3",
    "Data 4",
    "Data 5",
    "Data 6",
    "Data 7",
    "Data 8",
    "Data 9",
    "Data 10",
    "Data 11",
    "Data 14",
    "Data 16",
]


def report_symbols() -> dict:
    """
    Get a list of all the symbols in the Weekly Report
    :return:
    """
    ex = excel_scraper.read_excel_file(fileloc)
    symbols = {}
    for tab in sheets_to_parse:
        if "Data" in tab:
            d = ex.parse(tab, skiprows=[0]).head(1).iloc[0].to_dict()
            symbols = {**symbols, **d}
    return symbols


def read_canada_imports():
    df = excel_scraper.read_table(
        canada_importa_file_loc, sheet_name="Data 1", skiprows=(0, 2), index_col=0
    )
    return df


def read_report() -> pd.DataFrame:
    """
    Read all the timeseries in the report
    :return:
    """
    ex = excel_scraper.read_excel_file(fileloc)
    dfs = []
    for tab in sheets_to_parse:
        if "Data" in tab:
            d = ex.parse(tab, skiprows=[0, 2], index_col=0)
            d = d["2000":]
            # if tab in ['Data 1', 'Data 2', 'Data 6']:
            d = d / 1000
            if tab == "Data 14":
                # d.rename(columns={'WDIRPUS2':'WDIRPUS2_4wk', 'WKJRPUS2':'WKJRPUS2_4wk'}, inplace=True)

                d.rename(columns={"WDIRPUS2": "WDIRPUS2_4wk"}, inplace=True)
                d = d["WDIRPUS2_4wk"]
            if tab == "Data 16":
                d.rename(
                    columns={"W_EPOOXE_YOP_NUS_MBBLD": "W_EPOOXE_YOP_NUS_MBBLD_4wk"},
                    inplace=True,
                )
                d = d["W_EPOOXE_YOP_NUS_MBBLD_4wk"]

            dfs.append(d)
        else:
            d = ex.parse(tab)
            ind_row = 27
            ind_col = 2
            # Check that the values we have match the location of 'Release Date:' in the file
            rl_date_ind = int(d[d.eq("Release Date:").any(1)].index.values)
            if rl_date_ind == ind_row:
                release_date = pd.to_datetime(d.iloc[ind_row][ind_col], dayfirst=False)
            else:
                raise ValueError("Row indexes do not match")
    res = pd.concat(dfs, axis=1)

    res["Gas Yield"] = (res["W_EPM0F_YPR_NUS_MBBLD"] / res["WCRRIUS2"]) * 100
    res["Non Oxy Yield"] = (
        (res["W_EPM0F_YPR_NUS_MBBLD"] - res["W_EPOOXE_YOP_NUS_MBBLD"]) / res["WCRRIUS2"]
    ) * 100
    res["Dist Yield"] = (res["WDIRPUS2"] / res["WCRRIUS2"]) * 100

    res["mogas_dc"] = (res["WGTSTUS1"] / res["WGFUPUS2"]).rolling(window=4).mean()
    res["distillate_dc"] = (res["WDISTUS1"] / res["WDIRPUS2"]).rolling(window=4).mean()
    res["jet_dc"] = (res["WKJSTUS1"] / res["WKJUPUS2"]).rolling(window=4).mean()

    res = pdu.mergets(res, read_canada_imports())

    return res, release_date


def gen_page(
    title: str, template: str, filename: str = None, report_data: pd.DataFrame = None
):
    data = {"name": title, "title": title, "eia_url": eia_url}

    if report_data is None:
        report_data = read_report()
    data["report"], data["release_date"] = report_data

    return ju.render_html(
        data,
        template=template,
        filename=filename,
        package_loader_name="oilanalytics.eia",
        template_globals={"cu": cu},
    )


def gen_summary_charts(filename, report_data=None):
    data = {
        "name": "DOE Weekly Quick Report",
        "title": "DOE Weekly Quick",
        "eia_url": eia_url,
    }

    if report_data is None:
        report_data = read_report()
    data["report"] = report_data

    return ju.render_html(
        data,
        "doe_weekly_em.html",
        filename=filename,
        package_loader_name="oilanalytics.eia",
        template_globals={"cu": cu},
    )


def gen_and_send_email(
    report_data=None,
    sender_email: str = None,
    receiver_email: str = None,
    dashboard_link: str = None,
):
    data = {
        "name": "DOE Weekly Petroleum Report",
        "title": "DOE Weekly",
        "eia_url": eia_url,
    }
    if report_data is None:
        report_data = read_report()[0]

    summary_table_items = {
        "WCESTUS1": "Crude Stocks",
        "W_EPC0_SAX_YCUOK_MBBL": "Cushing Stocks",
        "WGTSTUS1": "Mogas Stocks",
        "WDISTUS1": "Distillate Stocks",
        "WGFUPUS2": "Mogas Demand",
        "WDIUPUS2": "Distillate Demand",
        "WCRFPUS2": "Crude Supply",
        "WCREXUS2": "Crude Exports",
        "WCRRIUS2": "Runs",
    }

    df_sum = report_data[summary_table_items.keys()].rename(columns=summary_table_items)
    data["summary_table"] = cu.gen_wow_summary_table(df_sum)

    for key, title in summary_table_items.items():
        data[title] = cu.seas_chart_weekly(df=report_data, series=key, title=title)

    if dashboard_link:
        data["dashboard_link"] = f'<a href="{dashboard_link}">Link to Dashboard</a>'

    compose_and_send_jinja_report(
        "DOE EIA Weekly Report",
        data=data,
        template="doe_weekly_email.html",
        package_loader_name="oilanalytics.eia",
        receiver_email=receiver_email,
        sender_email=sender_email,
    )


def extract_release_date(url: str) -> datetime.date:
    """
    Given a url, extract the release date to allow comparisons between runs and detect if report has been updated
    :param url:
    :return:
    """

    http = urllib3.PoolManager(cert_reqs="CERT_NONE", assert_hostname=False)
    pg = http.request("GET", url)

    # Get html docstring
    pg_data = str(pg.data)
    x = re.search("\d{1,2}-[a-zA-Z]{3}-\d{4}", pg_data)
    if x:
        release_date = datetime.strptime(x[0], "%d-%b-%Y")
        return release_date


if __name__ == "__main__":
    gen_page(
        title="DOE Weekly Report",
        template="templates/doe_weekly_summary.html",
        filename=r"summary.html",
    )
    gen_page(
        title="DOE Weekly Report - Refineries",
        template="doe_weekly_refineries.html",
        filename=r"refineries.html",
    )
    gen_page(
        title="DOE Weekly Report - Distillates",
        template="doe_weekly_distillates.html",
        filename=r"distillates.html",
    )
    gen_page(
        title="DOE Weekly Report - Jet",
        template="doe_weekly_jet.html",
        filename=r"jet.html",
    )
    gen_page(
        title="DOE Weekly Report - Fuel",
        template="doe_weekly_fuel.html",
        filename=r"fuel.html",
    )
    gen_page(
        title="DOE Weekly Report - LPG",
        template="doe_weekly_lpg.html",
        filename=r"lpg.html",
    )
    gen_page(
        title="DOE Weekly Report - Ethanol",
        template="doe_weekly_ethanol.html",
        filename=r"ethanol.html",
    )
    gen_page(
        title="DOE Weekly Report - Crude",
        template="doe_weekly_crude.html",
        filename=r"crude.html",
    )
    gen_page(
        title="DOE Weekly Report - Gasoline",
        template="doe_weekly_gasoline.html",
        filename=r"gasoline.html",
    )



import urllib.request

import pandas as pd
from bs4 import BeautifulSoup
from dotenv import load_dotenv
from excel_scraper import excel_scraper

# Load environmental variables from '.env' file.
load_dotenv()


def get_file_url():
    html_page = urllib.request.urlopen(
        "https://www.opec.org/opec_web/en/publications/338.htm"
    )
    soup = BeautifulSoup(html_page, "html.parser")
    links = [
        x.get("href") for x in soup.findAll("a") if x.get("href").endswith(".xlsx")
    ]
    link = links[0]
    link = "%s/%s" % ("https://www.opec.org", link)
    return link


def read_table1(fileloc=None):
    # todo grab file directly from website
    if not fileloc:
        fileloc = get_file_url()
    df = excel_scraper.read_table(
        fileloc, sheet_name="Table 11 - 1", skiprows=[0, 1, 2, 3]
    )

    colhead = "World oil demand and supply balance"
    # find col after colhead to help filter for main sections
    col_after = df.columns[list(df.columns.values).index(colhead) + 1]

    df["Prefix"] = df.apply(
        lambda x: x[colhead] if pd.isnull(x[col_after]) else None, 1
    ).ffill()
    df["Concat"] = df.apply(
        lambda x: "%s-%s" % (x["Prefix"], x[colhead])
        if pd.notnull(x[col_after])
        else x["Prefix"],
        1,
    ).ffill()
    df = df.set_index("Concat")
    df = df.filter(regex="\d\d\d\d|\d[Q]\d\d", axis=1)  # filter only on valid dates
    df = df.T
    df = df.dropna(how="all", axis=1)
    return df


cols = {
    "World demand-Total OECD": "OECD",
    "World demand-Total Non-OECD": "Non-OECD",
    "World demand-(a) Total world demand": "Demand",
    "Non-OPEC liquids production-Total Non-OPEC production": "Non-OPEC supply",
    "Non-OPEC liquids production-Americas": "North America",
    "Non-OPEC liquids production-  of which US": "USA",
    "NON-OECD SUPPLY-FSU": "FSU",
    "OPEC-NGLs": "OPEC NGLs/Condensates",
    "Non-OPEC liquids production-OPEC crude oil production (secondary sources)": "OPEC crude",
    "Non-OPEC liquids production-Balance (stock change and miscellaneous)": "Stock change",
}


def comparable_snd_table():
    df = read_table1()
    df = df.rename(columns=cols)
    df = df[df.index.isin([x for x in df.index if not isinstance(x, int)])]
    df.index = pd.PeriodIndex(df.index, freq="Q")
    return df



import pytest

from oilanalytics.highfreq import ara, singapore


@pytest.mark.skip("Eikon required")
def test_ara_data():
    res = ara.ara_data(start_date="2022-01-01")
    assert "Jet" in res.columns
    assert res["Jet"].loc["2022-01-06"] == pytest.approx(7.06, 0.01)
    # Note freq might not be consistent across time, so this won't be true across history
    # self.assertEqual(pd.infer_freq(res.index), 'W-THU')


@pytest.mark.skip("Eikon required")
def test_sing_data():
    res = singapore.singapore_data(start_date="2022-01-01")
    assert "LightDistillates" in res.columns



import logging as logger
from datetime import datetime
from oilanalytics.utils import chartutils as cu
from commodplot import jinjautils as ju
from pyenergyaspects import eaapi

fsu_stat_ids = [
    9042,
    9063,
    9092,
    10284,
    3956,
    3957,
    3958,
    3959,
    3960,
    4758,
    1294,
    8989,
    8991,
    3949,
    3952,
    3954,
    3955,
]


def generate_page(
    file_path: str = None,
    title_url: str = None,
):
    data = {
        "name": "FSU Stats",
        "title": "FSU Stats",
    }

    date_from = datetime(2010, 1, 1)

    data["df"] = eaapi.read_timeseries(dataset_id=fsu_stat_ids, date_from=date_from)

    data["title_url"] = title_url

    logger.info(f"generated page to {file_path}")
    return ju.render_html(
        data=data,
        template="fsu_stats.html",
        filename=file_path,
        package_loader_name="oilanalytics.energyaspects",
        template_globals={"cu": cu},
    )


if __name__ == "__main__":
    generate_page(file_path="fsu_stats.html")



from datetime import date

import pandas as pd

from oilanalytics.eia import weeklypetreport


def test_read_report():
    df, rd = weeklypetreport.read_report()
    assert isinstance(df, pd.DataFrame)
    assert isinstance(rd, date)


def test_gen_page():
    res = weeklypetreport.gen_page("Test", template="templates/doe_weekly_summary.html")
    assert res is not None



import logging as logger
import typing as t
from datetime import date

import cachetools.func
import pandas as pd
import plotly.graph_objects as go
from commodplot import commodplot as cpl
from commodplot import jinjautils as ju
from commodutil import forwards
from plotly.subplots import make_subplots
from pylim import lim

futures_dict = {
    "FB": "Brent",
    "CL": "WTI",
    "DME.OQ": "Dubai",
    "RB.UNL": "RBOB",
    "HO": "HO",
    "FP": "ULSD",
}

crude_futures = {"FB": "Brent", "CL": "WTI", "DME.OQ": "Dubai"}

default_cust_charts = {"JunJun": "JunJun", "DecDec": "DecDec"}


@cachetools.func.ttl_cache(ttl=100 * 60)
def get_contracts(
    limsymbol: str, start_year: int = 2019, end_year: int = 2024
) -> pd.DataFrame:
    contracts = lim.contracts(
        limsymbol,
        start_year=2019,
        end_year=end_year,
        start_date=pd.to_datetime(str(start_year)),
    )
    contracts = lim.limutils.convert_lim_contracts_to_datetime(contracts)

    return contracts


def generate_oi_vol_plot(df: pd.DataFrame):
    # Create subplots and mention plot grid size
    fig = make_subplots(
        rows=3,
        cols=1,
        shared_xaxes=True,
        vertical_spacing=0.03,
        subplot_titles=("Price", "Volume", "OpenInterest"),
        row_width=[0.33, 0.33, 0.33],
    )

    # Plot OHLC on 1st row
    fig.add_trace(
        go.Candlestick(
            x=df.index,
            open=df["Open"],
            high=df["High"],
            low=df["Low"],
            close=df["Close"],
            name="Price",
            showlegend=False,
        ),
        row=1,
        col=1,
    )

    # Bar trace for volumes on 2nd row without legend
    fig.add_trace(go.Bar(x=df.index, y=df["TotalVol"], showlegend=False), row=2, col=1)
    fig.add_trace(go.Bar(x=df.index, y=df["TotalOI"], showlegend=False), row=3, col=1)

    # Do not show OHLC's rangeslider plot
    fig.update(layout_xaxis_rangeslider_visible=False)
    fig.update_layout(height=1000)
    for subx in range(0, 3):
        fig.layout.annotations[subx].update(x=0.025)
    return fig


def generate_symbol_page(
    limsymbol: str, symbolname: str, cust_charts: dict = None, file_path: str = None
):
    df = lim.series(limsymbol, start_date=pd.to_datetime("2015"))
    candle = lim.candlestick_data(
        limsymbol, additional_columns=("TotalOI", "TotalVol"), days=440
    )

    data = {
        "name": symbolname,
        "title": symbolname,
        "candle_chart": cpl.candle_chart(candle.tail(90), title=symbolname),
        "seas_chart": cpl.seas_line_plot(df, inc_change_sum=False, title="Seasonal"),
    }

    all_curve_dates = df.dropna()
    asked_indexes = [-i for i in (1, 2, 5, 10, 20, 30) if i <= len(all_curve_dates)]
    curve_dates = list(all_curve_dates.iloc[asked_indexes].index)
    data["curve_hist"] = cpl.forward_history_plot(
        lim.curve(limsymbol, curve_dates=curve_dates), title="Curve History"
    )

    structure = futures_structure(
        start_date="date is within 3650 days", symbol=limsymbol, convert_to_bbls=False
    )
    structure = structure[
        [x for x in structure if "M1M2" in x or "M1M6" in x or "M1M12" in x]
    ]

    data["structure"] = cpl.line_plot(structure.tail(60), title="Structure")
    data["m1m2"] = cpl.seas_line_plot(
        structure[[x for x in structure if "M1M2" in x]], title="M1-M2"
    )
    data["m1m6"] = cpl.seas_line_plot(
        structure[[x for x in structure if "M1M6" in x]], title="M1-M6"
    )
    data["m1m12"] = cpl.seas_line_plot(
        structure[[x for x in structure if "M1M12" in x]], title="M1-M12"
    )

    contracts = get_contracts(limsymbol=limsymbol)
    for spread in [
        "janfeb",
        "febmar",
        "marapr",
        "aprmay",
        "mayjun",
        "junjul",
        "julaug",
        "augsep",
        "sepoct",
        "octnov",
        "novdec",
        "decjan",
    ]:
        d = forwards.spread_combination(contracts=contracts, combination_type=spread)
        data[spread] = cpl.reindex_year_line_plot(d, title=spread)

    if cust_charts:

        index = 1
        for cust_chart in cust_charts:
            d = forwards.spread_combination(contracts, cust_chart)
            f = cpl.reindex_year_line_plot(d, title=cust_charts[cust_chart])
            data["cust_ch%s" % index] = f
            index += 1

    data["oi_vol"] = generate_oi_vol_plot(candle)

    if file_path:
        logger.info(f"generated {symbolname} page to {file_path}")
    return ju.render_html(
        data,
        "symbol.html",
        filename=file_path,
        package_loader_name="oilanalytics.prices",
    )


def futures_structure(
    start_date: t.Optional[t.Union[str, date]] = "date is within 5 days",
    convert_to_bbls: bool = True,
    symbol: str = None,
) -> pd.DataFrame:
    dfs = {}
    loop = futures_dict
    if symbol:
        loop = {symbol: symbol}

    for symbol in loop.keys():
        df = lim.continuous_futures_rollover(
            symbol, months=["M1", "M2", "M6", "M12"], start_date=start_date
        )
        if convert_to_bbls:
            if symbol == "FP":
                df = df / 7.45
            if symbol in ("RB.UNL", "HO"):
                df = df * 0.42

        df[f"{symbol}_M1M2"] = df[f"{symbol}_M1"] - df[f"{symbol}_M2"]
        df[f"{symbol}_M1M6"] = df[f"{symbol}_M1"] - df[f"{symbol}_M6"]
        df[f"{symbol}_M1M12"] = df[f"{symbol}_M1"] - df[f"{symbol}_M12"]
        dfs[symbol] = df

    res = pd.concat(list(dfs.values()), axis=1)
    return res


def generate_structure_page(
    start_date: t.Optional[t.Union[str, date]] = date(2018, 1, 1), file_path: str = None
):
    df = futures_structure(start_date=start_date)

    data = {
        "name": "Structure",
        "title": "Structure",
    }

    data["m1m2"] = cpl.line_plot(df[[x for x in df.columns if "M1M2" in x]])
    data["m1m6"] = cpl.line_plot(df[[x for x in df.columns if "M1M6" in x]])
    data["m1m12"] = cpl.line_plot(df[[x for x in df.columns if "M1M12" in x]])
    if file_path:
        logger.info(f"generated structure page to {file_path}")
    return ju.render_html(
        data=data,
        template="structure.html",
        filename=file_path,
        package_loader_name="oilanalytics.prices",
    )


if __name__ == "__main__":
    # print(get_prices(crude_futures))
    # generate_symbol_page('FB', 'Brent', cust_charts={'JunJun': 'JunJun', 'DecDec': 'DecDec'}, file_path='Brent.html')
    # df = futures_structure()
    generate_structure_page(file_path="Structure.html")



import pandas as pd
from commodplot import commodplot as cpl
from commodplot import commodplottable as cplt

from oilanalytics.balances import balance_utils as bu


def format_chart(plotly_plot, chart_type=None, title=None):
    """apply standard formatting to a plotly plot"""

    plotly_plot.update_layout(
        {
            # shrink margins to maximise plot size
            "margin": {"l": 0, "r": 0, "t": 40, "b": 0}
        }
    )

    if chart_type == "seasonal":
        plotly_plot.update_layout(
            {
                # reverse traces to get latest dates at the top
                "legend": {"traceorder": "reversed", "font": {"size": 12}},
            }
        )
    elif chart_type == "table":
        format_dict = {"height": 30, "font": {"size": 20}}
        plotly_plot.update_traces({"cells": format_dict, "header": format_dict})
        plotly_plot.update_layout({"title": {"text": title, "font": {"size": 20}}})

    return plotly_plot


def seas_chart(
    df: pd.DataFrame,
    series: str,
    title: str,
    title_url: str = None,
    histfreq: str = None,
):
    if title_url:
        title_url_f = title_url % series
        title = f'<a href="{title_url_f}">{title}</a>'

    fig = cpl.seas_line_plot(
        df[series],
        title=title,
        shaded_range=5,
        average_line=5,
        histfreq=histfreq,
        visible_line_years=3,
    )

    return fig


def seas_chart_weekly(df: pd.DataFrame, series: str, title: str, title_url: str = None):
    return seas_chart(
        df=df, series=series, title=title, title_url=title_url, histfreq="W"
    )


def gen_wow_summary_table(df_input, precision=2, single_date=False):
    """
    generic method to create a summary table for individual location page
    :param df_input: DataFrame to summarise. Must have datetime index
    :return: html table containing summary data
    """
    df_sum = bu.calculate_change_from_prev_week(df_input)
    # format the DataFrame
    df_sum = df_sum.round(2)
    df_sum.index.rename("Product", inplace=True)
    df_sum = df_sum[
        [df_sum.columns[1], df_sum.columns[0]] + df_sum.columns[2:].tolist()
    ]  # swap first 2 columns
    df_sum.columns.name = None
    df_sum.index.name = None
    if single_date:
        df_sum.rename(columns={df_sum.columns[1]: "Last Week"}, inplace=True)
    get_summary_table = cplt.generate_table(
        df_sum, precision=precision, accounting_col_columns=["Change", "%"]
    )

    return get_summary_table



import pandas as pd
from commodutil import dates


def color_negative_red(value):
    """
    Colors elements in a dateframe
    green if positive and red if
    negative. Does not color NaN
    values.
    """

    if value < 0:
        color = "red"
    elif value > 0:
        color = "green"
    else:
        color = "black"

    return "color: %s" % color


def color_negative_red_series(series):
    return series.apply(lambda x: color_negative_red(x), 1)


def color_negative_red_row(series):
    return series.apply(lambda x: color_negative_red(x), 0)


def style_quarter_summary_table(df, accounting_col=None):
    style_td = dict(
        selector="td", props=[("border-style", "solid"), ("border-width", "1px")]
    )
    style_th = dict(
        selector="th", props=[("border-style", "solid"), ("border-width", "1px")]
    )

    year_subset = [str(x) for x in list(set(dates.find_year(df).values()))]
    dfs = (
        df.style.format("{:.2f}")
        .apply(color_negative_red_row)  # Todo apply to certain rows only
        .set_properties(**{"background-color": "lightblue"}, axis=1, subset=year_subset)
        .set_table_styles([style_th, style_td])
    )
    return dfs.render()


def quarter_summary_format(df, accounting_col=None):
    """
    Given a dataframe where the index is PeriodIndex(period[Q-DEC]),
    format dataframe using pandas styler to:
    - have yearly mean inserted between quarters
    - format to 2dp
    - highlight +/- using red/green colours for selected rows
    - transpose table horizontally

    :param df:
    :param accounting_col:
    :return:
    """
    df = add_year_agg_to_qtr_index(df)
    df.index = df.index.astype(
        str, copy=False
    )  # requires string for style to apply slicers
    df.columns.name = None
    df = df.T  # make it horizontal table
    # df = df.replace(np.nan, None)

    dfs = style_quarter_summary_table(df, accounting_col)
    return dfs


def add_year_agg_to_qtr_index(df):
    """
    Given a dataframe of PeriodIndex('Q') add Yearly averages and return
    with yearly avg inserted into the appropriate rows
    :param df:
    :return:
    """

    y = df.groupby(pd.Grouper(freq="Y")).mean()  # average the qtrs
    dfs = []
    for x, year in y.iterrows():
        year_name = year.name
        if isinstance(year_name, pd.Timestamp):
            year_name = year_name.year
        d = df.loc[str(year_name)]
        d = pd.concat([d, pd.DataFrame(year).T])
        dfs.append(d)

    res = pd.concat(dfs)
    return res


def calculate_change_from_prev_week(df_input):
    """
    calculates absolute and % change from previous week for a given DataFrame
    """
    df_change = df_input.sort_index().tail(2)
    if isinstance(df_change.index, pd.DatetimeIndex):
        df_change.index = df_change.index.strftime("%d-%b")
    df_change = df_change.T
    df_change["Change"] = (
        df_change[df_change.columns[1]] - df_change[df_change.columns[0]]
    )
    df_change["%"] = 100 * df_change["Change"] / df_change[df_change.columns[0]]
    return df_change



import logging
from datetime import datetime

import pandas as pd
from pyenergyaspects import eaapi as energyaspects_api

from oilanalytics.balances import balance_utils as bu

products_region_list = [
    "WORLD",
    "NA",
    "US",
    "PADD_1",
    "PADD_3",
    "EUR",
    "ME",
    "AP",
    "CN",
    "IN",
    "",
]


def add_formats(workbook):
    # https://xlsxwriter.readthedocs.io/working_with_colors.html
    header_date_format = workbook.add_format(
        {"num_format": "mmm-yy", "bg_color": "blue", "font_color": "white"}
    )
    header_qtr_format = workbook.add_format({"bg_color": "blue", "font_color": "white"})
    mbd_format = workbook.add_format({"num_format": "##,##0.00,"})
    mbd_format_forecast = workbook.add_format(
        {"num_format": "##,##0.00,", "bg_color": "lime"}
    )
    mbd_format_accounting = workbook.add_format(
        {"num_format": "[Color10]##,##0.00,;[Red]-##,##0.00,"}
    )

    formats = {
        "header_date_format": header_date_format,
        "header_qtr_format": header_qtr_format,
        "mbd_format": mbd_format,
        "mbd_format_forecast": mbd_format_forecast,
        "mbd_format_accounting": mbd_format_accounting,
    }
    return formats


def add_header(header, worksheet, pos_params, formats):
    # monthly header
    for count, date in header.iterrows():
        worksheet.write_datetime(
            0,
            pos_params["col_mth_start"] + count,
            datetime.date(date.iloc[0]),
            cell_format=formats["header_date_format"],
        )

    # quarterly header
    col_counter = pos_params["col_qtr_start"]
    for year in range(pos_params["start_year"], pos_params["end_year"] + 1):
        for i in ["1Q", "2Q", "3Q", "4Q", "Cal"]:
            period = "%s%s" % (i, str(year)[-2:]) if i[1] == "Q" else str(year)
            worksheet.write(0, col_counter, period, formats["header_qtr_format"])
            col_counter += 1


def add_forecast_colours_row(
    worksheet,
    df_row: pd.Series,
    row_number: int,
    col_number: int,
    start_date: datetime.date,
    format,
):
    for date, val in df_row.iteritems():
        if date >= start_date:
            worksheet.write_number(
                row=row_number, col=col_number, number=val, cell_format=format
            )
        col_number += 1


def add_forecast_colours(worksheet, df: pd.DataFrame, pos_params: dict, formats: dict):
    row_counter = pos_params["last_row"]
    for col in df.columns:
        df_row = df[col]
        fcstart = energyaspects_api.filter_meta_data(
            df, column=col, value="forecast_start_date"
        )
        if fcstart:
            fcstart = pd.to_datetime(fcstart)
            if fcstart > df.index[0]:
                add_forecast_colours_row(
                    worksheet,
                    df_row=df_row,
                    row_number=row_counter,
                    col_number=pos_params["col_mth_start"],
                    start_date=fcstart,
                    format=formats["mbd_format_forecast"],
                )

        row_counter += 1


def add_row_section(df, writer, worksheet, pos_params, formats):
    """
    Given a dataframe with timeseries data, transform and place into Excel.
    Apply formatting and showing summary quarter/calendar data
    :param df:
    :param writer:
    :param worksheet:
    :param pos_params:
    :param formats:
    :return:
    """
    df.T.to_excel(
        writer,
        sheet_name=worksheet.name,
        header=False,
        index=False,
        startrow=pos_params["last_row"],
        startcol=pos_params["col_mth_start"],
    )

    df_qtr = df.resample("QS").mean()
    df_qtr = bu.add_year_agg_to_qtr_index(df_qtr)
    df_qtr.T.to_excel(
        writer,
        sheet_name=worksheet.name,
        header=False,
        index=False,
        startrow=pos_params["last_row"],
        startcol=pos_params["col_qtr_start"],
    )

    cols = df.T.index
    cols = pd.DataFrame.from_records(cols.values, columns=cols.names)
    cols.to_excel(
        writer,
        sheet_name=worksheet.name,
        header=False,
        index=False,
        startrow=pos_params["last_row"],
    )

    # add red/green accounting colours on snd rows
    snd_row = cols[cols["aspect"] == "snd"].index[0]
    worksheet.set_row(
        row=pos_params["last_row"] + snd_row,
        cell_format=formats["mbd_format_accounting"],
    )

    # add forecast start colours
    add_forecast_colours(worksheet, df, pos_params=pos_params, formats=formats)

    # keep track of which row we are on by setting last_row as this section + 1
    pos_params["last_row"] = pos_params["last_row"] + len(df.T) + 1


def get_sheet_positional_params(df):
    start_year, end_year = df.index[0].year, df.index[-1].year
    years_span = (end_year - start_year) + 1
    col_qtr_start = 3
    col_mth_start = col_qtr_start + (5 * years_span) + 1
    data_columns = "D:BZ"

    return {
        "start_year": start_year,
        "end_year": end_year,
        "years_span": years_span,
        "col_qtr_start": col_qtr_start,
        "col_mth_start": col_mth_start,
        "data_columns": data_columns,
    }


def generate_product_sheet(
    commod: str, date_from: datetime.date, writer, formats: dict
):
    # need to generate worksheet using pandas
    pd.DataFrame([commod]).to_excel(
        writer, sheet_name=commod, header=False, index=False
    )

    worksheet = writer.sheets[commod]
    # Freeze first 2 rows or columns ABC
    worksheet.freeze_panes(2, 3)

    source = "Energy Aspects"
    column_format = {
        "format": ("extract_region", "aspect", "source"),
        "multiindex": True,
    }
    # df = energyaspects_api.read_commod_balance(commod=commod, geography='US', column_format=column_format,
    #                                            date_from=date_from)

    df = energyaspects_api.read_commod_balance(
        commod=commod, column_format=column_format, date_from=date_from
    )

    # ea don't have a source filter
    df = df.loc[:, (slice(None), slice(None), "Energy Aspects")]
    pos_params = get_sheet_positional_params(df)
    worksheet.set_column(pos_params["data_columns"], cell_format=formats["mbd_format"])

    add_header(
        pd.DataFrame(df.index), worksheet, pos_params=pos_params, formats=formats
    )

    pos_params["last_row"] = 2

    for region in products_region_list:
        try:
            dfi = df.loc[:, (region, ("supply", "demand"))]
            dfi.loc[:, (region, "snd", source)] = (
                dfi[(region, "supply", source)] - dfi[(region, "demand", source)]
            )
            add_row_section(
                dfi,
                writer=writer,
                worksheet=worksheet,
                pos_params=pos_params,
                formats=formats,
            )
        except ValueError as ex:
            logging.info("Unable to add section for %s: %s" % (region, ex))
        except KeyError as ex:
            logging.info("Unable to add section for %s: %s" % (region, ex))


def generate_balance():
    ts = datetime.now().strftime("%Y%m%d_%H%Y%M")
    workbookname = f"balance_{ts}.xlsx"

    writer = pd.ExcelWriter(workbookname, engine="xlsxwriter")
    workbook = writer.book
    formats = add_formats(workbook)

    date_from = pd.to_datetime("2020-01-01")

    for commod in ["gasoline"]:
        generate_product_sheet(
            commod, writer=writer, date_from=date_from, formats=formats
        )

    writer.save()
    return workbookname


if __name__ == "__main__":
    r = generate_balance()
    import os

    os.system("start Excel " + r)



import pandas as pd

from oilanalytics.eia import eia as eia

# https://www.eia.gov/opendata/qb.php?category=829716

categs = [
    829747,  # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Consumption
    829748,  # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Inventories
    829751,
    # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Production > Non-OPEC
    1039874,
    # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Production > OPEC
]


def get_data(freq="M", last=10):
    series = eia.child_series(categs, freq_filter=freq)
    series_ids = [x["series_id"] for x in series]
    df = eia.get_data(series_ids, last)
    return df


cols = {
    "STEO.PATC_OECD.Q": "OECD",
    "STEO.PATC_NON_OECD.Q": "Non-OECD",
    "STEO.PATC_WORLD.Q": "Demand",
    "NON-OECD SUPPLY-Total Non-OPEC Supply2": "Non-OPEC supply",
    "OECD SUPPLY-Americas4": "North America",
    "NON-OECD SUPPLY-FSU": "FSU",
    "OPEC-NGLs": "OPEC NGLs/Condensates",
    "STEO.PAPR_OPEC.Q": "OPEC crude",
    "STOCK CHANGES AND MISCELLANEOUS-Total Stock Ch. & Misc": "Stock change",
}


def comparable_snd_table():
    df = get_data(freq="Q", last=10)
    df = df.rename(columns=cols)
    df.index = pd.PeriodIndex(df.index, freq="Q")
    return df



import pandas as pd
import pytest
from oilanalytics.eia import eia, steo


@pytest.mark.skip(reason="EAIPY DOESN'T SUPPORT V2 OF EIA API")
def test_get_data_1():
    res = eia.get_data(["STEO.PATC_NON_OECD.M", "STEO.PATC_OECD.M"])
    assert "STEO.PATC_OECD.M" in res.columns
    assert "STEO.PATC_NON_OECD.M" in res.columns
    assert isinstance(res.index, pd.DatetimeIndex)


@pytest.mark.skip(reason="EAIPY DOESN'T SUPPORT V2 OF EIA API")
def test_get_data_2():
    res = eia.get_data(["STEO.PATC_NON_OECD.Q", "STEO.PATC_OECD.M"])
    assert "STEO.PATC_OECD.M" in res.columns
    assert isinstance(res.index, pd.DatetimeIndex)


@pytest.mark.skip(reason="EAIPY DOESN'T SUPPORT V2 OF EIA API")
def test_child_series():
    categs = [
        829747,  # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Consumption
        829748,  # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Inventories
        829751,
        # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Production > Non-OPEC
        1039874,
        # EIA Data Sets > Short-Term Energy Outlook > International Petroleum and Other Liquids > Production > OPEC
    ]
    res = eia.child_series(categs, freq_filter="M")
    assert "STEO.PATC_NON_OECD.M" in [x["series_id"] for x in res]


@pytest.mark.skip(reason="EAIPY DOESN'T SUPPORT V2 OF EIA API")
def test_steo_get_data():
    res = steo.get_data()
    assert "STEO.PATC_NON_OECD.M" in res.columns



global_oil_cols = [
    "Demand",
    "OECD",
    "Non-OECD",
    "Non-OPEC supply",
    "North America",
    "FSU",
    "OPEC NGLs/Condensates",
    "OPEC crude",
    "Stock change",
]

global_oil_basic_summary_cols = [
    "Demand",
    "Non-OPEC supply",
    "OPEC crude",
    "OPEC NGLs/Condensates",
    "Stock change",
]



import pandas as pd
import pytest

from oilanalytics.iea import omr


@pytest.mark.skip("TODO: Mock IEA ZIP file")
def test_read_world_snd_table1():
    res = omr.read_world_snd_table1()
    assert isinstance(res, pd.DataFrame)


@pytest.mark.skip("TODO: Mock IEA ZIP file")
def test_comparable_snd_table():
    res = omr.comparable_snd_table()
    assert isinstance(res, pd.DataFrame)
    assert isinstance(res.index, pd.PeriodIndex)



from oilanalytics.prices import futures_prices


def test_futures_structure():
    res = futures_prices.futures_structure(start_date="date is within 5 days")
    assert "FB_M1" in res.columns


def test_gen_symbol_page():
    res = futures_prices.generate_symbol_page("FB", symbolname="Brent")
    assert res is not None


def test_gen_structure_page():
    res = futures_prices.generate_structure_page()
    assert res is not None



import pytest

from oilanalytics.energyaspects import china_stats, fsu_stats


def test_china_stats():
    res = china_stats.generate_page()
    assert res is not None


@pytest.mark.skip("No longer permissioned on EA API")
def test_fsu_stats():
    res = fsu_stats.generate_page()
    assert res is not None



import requests


def get_token(username, password):
    url = "https://www.jodidata.org/api/token"
    myobj = {"username": username, "password": password}

    x = requests.post(url, data=myobj)

    return x.text



import eikon as ek
import pandas as pd
from commodutil import convfactors

from oilanalytics.utils import eikonutils

start_date = "2010-01-01"


def ara_data(start_date=start_date, df: pd.DataFrame = None) -> pd.DataFrame:
    # note crude is from Genscape but subscription is required: 0#GNS-ARC-STOR
    rics = {
        "Gasoline": "STK-GL-ARA",
        "Naphtha": "STK-NAF-ARA",
        "Gasoil": "STK-GO-ARA",
        "FuelOil": "STK-FO-ARA",
        "Jet": "STK-JET-ARA",
    }

    if df is None:
        eikonutils.setup_eikon()
        df = ek.get_timeseries(
            list(rics.values()), fields="Close", start_date=start_date
        )
        df = df.rename(columns=dict((v, k) for k, v in rics.items()))

    if "Crude" in df.columns:
        df["Crude"] = df["Crude"] / 1000
    df["Gasoline"] = (
        convfactors.convert(
            df["Gasoline"], fromunit="kt", tounit="bbl", commodity="gasoline"
        )
        / 1000
    )
    df["Gasoil"] = (
        convfactors.convert(
            df["Gasoil"], fromunit="kt", tounit="bbl", commodity="diesel"
        )
        / 1000
    )
    df["Jet"] = (
        convfactors.convert(df["Jet"], fromunit="kt", tounit="bbl", commodity="jet")
        / 1000
    )
    df["FuelOil"] = (
        convfactors.convert(df["FuelOil"], fromunit="kt", tounit="bbl", commodity="fo")
        / 1000
    )
    df["Naphtha"] = (
        convfactors.convert(
            df["Naphtha"], fromunit="kt", tounit="bbl", commodity="naphtha"
        )
        / 1000
    )

    df.attrs["region"] = "ara"
    return df



import typing as t
from deprecated import deprecated
from functools import reduce



import pandas as pd
from dotenv import load_dotenv
from eiapy import Category, MultiSeries

# Load environmental variables from '.env' file.
load_dotenv()

@deprecated(reason="EIAPY doesn't support V2 API.")
def extract_dataframe(data: list, series_id: str, freq: str = "M") -> pd.DataFrame:
    df = pd.DataFrame(data, columns=["date", series_id]).set_index("date")
    f = None
    if freq == "M":
        f = "%Y%m"
    df.index = pd.to_datetime(df.index, format=f)
    return df


@deprecated(reason="EIAPY doesn't support V2 API.")
def get_data(series_ids: t.Tuple[str], last=10) -> pd.DataFrame:
    """
    Given a list of EIA series Id, call api, get the data and return as a DataFrame
    :param series_ids:
    :param last: how many data points going back are required
    :return:
    """
    res = MultiSeries(series_ids).last(last)
    dfs = [extract_dataframe(x["data"], x["series_id"], x["f"]) for x in res["series"]]
    dfs = reduce(
        lambda left, right: pd.merge(
            left, right, left_index=True, right_index=True, how="outer"
        ),
        dfs,
    )
    dfs.attrs["meta"] = {x["series_id"]: x for x in res["series"]}
    return dfs


@deprecated(reason="EIAPY doesn't support V2 API.")
def child_series(category_ids: t.Tuple[int], freq_filter: str = None) -> list:
    """
    Given a list of EIA category ids, get a list of series_ids that make up those categories
    :param category_ids:
    :param freq_filter: filter for frequency eg annual, monthly, quarterly
    :return:
    """
    a = [Category(x).get_info()["category"]["childseries"] for x in category_ids]
    a = [item for sublist in a for item in sublist]
    if freq_filter:
        a = [x for x in a if x["f"] == freq_filter]
    return a



import pandas as pd

from oilanalytics.opec import momr


def test_read_table1():
    res = momr.read_table1()
    assert isinstance(res, pd.DataFrame)


def test_comparable_snd_table():
    res = momr.comparable_snd_table()
    assert isinstance(res.index, pd.PeriodIndex)



import zipfile
from functools import lru_cache
from os import getenv

import pandas as pd
from dotenv import load_dotenv
from excel_scraper import excel_scraper

from oilanalytics.balances import balance_config

# Load environmental variables from '.env' file.
load_dotenv()

table1_section = [
    "OECD DEMAND",
    "NON-OECD DEMAND",
    "OECD SUPPLY",
    "NON-OECD SUPPLY",
    "OPEC",
    "STOCK CHANGES AND MISCELLANEOUS",
    "Memo items:",
]


@lru_cache(maxsize=10)
def read_zip(omr_zip_loc):
    return zipfile.ZipFile(omr_zip_loc)


def read_world_snd_table1(omr_zip_loc=None):
    if not omr_zip_loc:
        omr_zip_loc = getenv("IEA_OMR_LOC")
    z = read_zip(omr_zip_loc)
    fname = [x for x in z.filelist if x.filename.endswith("Table1.xls")][0]
    df = excel_scraper.read_table(z.open(fname), skiprows=[0, 1, 2, 3, 4, 5, 6])
    df["Prefix"] = df["Unnamed: 0"]
    df.loc[~df["Prefix"].isin(table1_section), "Prefix"] = None
    df["Prefix"] = df["Prefix"].fillna(method="ffill")
    df["Concat"] = df.apply(lambda x: "%s-%s" % (x["Prefix"], x["Unnamed: 0"]), 1)
    df = df.set_index("Concat")
    df = df.filter(regex="\d\d\d\d|\d[Q]\d\d", axis=1)  # filter only on valid dates
    df = df.T
    df = df.dropna(how="all", axis=1)
    df = df.rename(columns={x: x.strip() for x in df.columns})

    return df


cols = {
    "OECD DEMAND-Total OECD": "OECD",
    "NON-OECD DEMAND-Total Non-OECD": "Non-OECD",
    "NON-OECD DEMAND-Total Demand1": "Demand",
    "NON-OECD SUPPLY-Total Non-OPEC Supply2": "Non-OPEC supply",
    "OECD SUPPLY-Americas4": "North America",
    "NON-OECD SUPPLY-FSU": "FSU",
    "OPEC-NGLs": "OPEC NGLs/Condensates",
    "OPEC-Crude": "OPEC crude",
    "STOCK CHANGES AND MISCELLANEOUS-Total Stock Ch. & Misc": "Stock change",
}


def comparable_snd_table():
    df = read_world_snd_table1()
    df = df[df.index.str.contains("Q").fillna(False)]  # keep only quarters
    df.index = pd.PeriodIndex(df.index, freq="Q")
    df = df.rename(columns=cols)
    return df


def basic_comparison_table():
    df = comparable_snd_table()[balance_config.basic_summary_cols]
    return df



import logging

import eikon as ek
import pandas as pd
from commodplot import jinjautils as ju

from oilanalytics.utils import chartutils as cu
from oilanalytics.utils import eikonutils

start_date = "2010-01-01"


def singapore_data(start_date=start_date, df: pd.DataFrame = None) -> pd.DataFrame:
    rics = {
        "LightDistillates": "STKLD-SIN",
        "MiddleDistillates": "STKMD-SIN",
        "HeavyDistillates": "STKRS-SIN",
    }

    if df is None:
        eikonutils.setup_eikon()
        df = ek.get_timeseries(
            list(rics.values()), fields="Close", start_date=start_date
        )
        df = df.rename(columns=dict((v, k) for k, v in rics.items()))

    df = df / 1000  # kbbl to mbbl
    df.attrs["region"] = "singapore"
    return df


def generate_singapore_page(df_data=None, out_loc: str = None):
    logging.info("generating singapore page")
    if df_data is None:
        df_data = singapore_data()

    data = {"name": "Singapore", "title": "Singapore", "df_input": df_data}

    ju.render_html(
        data,
        "singapore.html",
        filename=out_loc,
        package_loader_name="oilfundamentals.highfrequency",
        template_globals={"cu": cu},
    )



import logging as logger
from datetime import datetime

from commodplot import jinjautils as ju
from pyenergyaspects import eaapi

from oilanalytics.utils import chartutils as cu

china_stat_ids = [
    3333,
    3334,
    3335,
    3336,
    3337,
    3325,
    3326,
    3329,
    3338,
    3339,
    3340,
    3341,
    3342,
    3343,
    3344,
    3345,
    3346,
    3347,
    3348,
    3349,
    3350,
    3351,
    3352,
    3330,
    3358,
    3359,
    3360,
    3361,
    3362,
    3363,
    3364,
    3365,
    3366,
    3367,
    3368,
    3369,
    3331,
    3332,
    1446,
    1481,
    1479,
    1480,
    3324,
]


def generate_page(
    file_path: str = None,
    title_url: str = None,
):
    data = {
        "name": "China Stats",
        "title": "China Stats",
    }

    date_from = datetime(2010, 1, 1)

    data["df"] = eaapi.read_timeseries(dataset_id=china_stat_ids, date_from=date_from)

    data["title_url"] = title_url

    logger.info(f"generated page to {file_path}")
    return ju.render_html(
        data=data,
        template="china_stats.html",
        filename=file_path,
        package_loader_name="oilanalytics.energyaspects",
        template_globals={"cu": cu},
    )


if __name__ == "__main__":
    generate_page(file_path="china_stats.html")



### Note the way this gets data via the EA excel file should be deprecated
### now that data is available via API


from os import getenv

import cachetools.func
import pandas as pd
import plotly.graph_objects as go
from commodplot import commodplot as cpl
from commodplot import jinjautils as ju
from commodutil import dates
from dotenv import load_dotenv
from excel_scraper import excel_scraper

from oilanalytics.utils import fileutils

# Load environmental variables from '.env' file.
load_dotenv()


def tar_to_timeseries(taramount, startdate, enddate, tarname=None):
    dr = pd.date_range("01/01/2015", "01/01/2025")
    ser = pd.Series(0, index=dr)
    ser[startdate:enddate] = taramount
    ser.name = tarname
    return ser


@cachetools.func.ttl_cache(maxsize=1, ttl=100 * 60)
def read_tars_excel():
    folder = getenv("EA_TARS_FOLDER")
    fileloc = fileutils.find_latest_file(folder)
    ex = excel_scraper.read_excel_file(fileloc)
    df = ex.parse("Refinery outages")
    df = df.rename(
        columns={x: x.lower() for x in df.columns}
    )  # make col headings lower case for consistency
    for col in [
        "region",
        "country",
        "planned/unplanned",
    ]:  # same with the column entries
        df[col] = df[col].str.lower()
    return df


@cachetools.func.ttl_cache(ttl=100 * 60)
def convert_table_to_tar_series(value_col="unit capacity"):
    tar_table = read_tars_excel()
    tarser = tar_table.apply(
        lambda x: tar_to_timeseries(
            x[value_col], x["start date"], x["end date"], x.name
        ),
        1,
    )
    tarser = tarser / 1000000
    return tarser


def tar_series(
    unittype="CDU",
    region=None,
    freq="MS",
    sum=True,
    value_col="unit capacity",
    planned=None,
):
    if region == "global":
        region = None

    df = read_tars_excel()
    tarser = convert_table_to_tar_series(value_col=value_col)
    df1 = df[df["unit type"] == unittype]
    if region is not None:
        df1 = df1[df1["region"] == region]

    if planned:
        if planned == "planned":
            df1 = df1[df1["planned/unplanned"] == "planned"]
            df1 = df1[df1["unplanned"] == "unplanned"]

    tarser = tarser[tarser.index.isin(df1.index)]  # filter tarser to same ids as df1
    if sum:
        tf = tarser[tarser.index.isin(df1.index)].sum()
        tf = tf.resample(freq).mean()
    else:
        tf = tarser[tarser.index.isin(df1.index)]

    return tf


def seas_chart(tarser, title):
    f = cpl.seas_line_plot(tarser, title=title, inc_change_sum=False)
    return f


def generate_ea_tars_page(out_loc=None):
    if not out_loc:
        out_loc = "TarsDashboard.html"

    data = {"name": "TARs Dashboard", "title": "Tars Dashboard"}
    data["global_cdu_tars"] = seas_chart(
        tar_series("CDU", region="global"), title="Global CDU Tars"
    )
    data["global_cdu_runs_impact"] = seas_chart(
        tar_series("CDU", region="global", value_col="estimated runs impact"),
        title="Estimates runs impact",
    )

    data["global_fcc_tars"] = seas_chart(
        tar_series("CDU", region="global"), title="Global FCC Tars"
    )
    data["global_hcu_tars"] = seas_chart(
        tar_series("HCU", region="global"), title="Global Hydrocrakcer Tars"
    )
    data["global_cok_tars"] = seas_chart(
        tar_series("COK", region="global"), title="Global Coker Tars"
    )
    ju.render_html(
        data,
        "tars_dashboard.html",
        filename=out_loc,
        package_loader_name="oilanalytics.energyaspects",
    )


def areachart(tarser, title):
    f = tarser.T[dates.curyear :]
    f = f.loc[:, (f != 0).any(axis=0)]  # remove all 0 columns
    fig = go.Figure()

    for col in f.columns:
        fig.add_trace(go.Scatter(x=f.index, y=f[col], stackgroup="one"))
    fig.update_layout(title=title)
    return fig


if __name__ == "__main__":
    # balance_comparisons()
    generate_ea_tars_page()
    # areachart()



import pandas as pd
from commodplot import jinjautils as ju
from commodplot.messaging import compose_and_send_jinja_report
from excel_scraper import excel_scraper
from datetime import datetime
import re
import requests
from oilanalytics.utils import chartutils as cu
from plotly.subplots import make_subplots
import plotly.graph_objects as go
from commodplot import commodplot as cpl
from bs4 import BeautifulSoup
import urllib.request as urllib2

filenames = ["dpr-data", "duc-data"]
fileloc = "https://www.eia.gov/petroleum/drilling//xls/%s.xlsx"

eia_webpage = "https://www.eia.gov/petroleum/drilling/"

sheets_to_graph = ["Bakken Region", "Eagle Ford Region", "Permian Region"]

all_sheets = [
    "Anadarko Region",
    "Appalachia Region",
    "Bakken Region",
    "Eagle Ford Region",
    "Haynesville Region",
    "Niobrara Region",
    "Permian Region",
]

def read_release_date():

    r = urllib2.urlopen(eia_webpage).read()
    x = []
    soup = BeautifulSoup(r, 'html.parser')
    result = soup.find_all('span', {'class': 'date'})[0].text.strip()
    if result:
        release_date = datetime.strptime(result, "%B %d, %Y")
    return release_date



def create_dualaxis_graph(df, title=None):
    # ensure that the DF is of format index / secondary axis col / primary axis col
    tf_list = [True, False]
    fig = make_subplots(specs=[[{"secondary_y": True}]])
    for col, tf in zip(df.columns, tf_list):
        if tf is False:
            fig.add_trace(go.Bar(x=df.index, y=df[col], name=col), secondary_y=tf)
        else:
            fig.add_trace(go.Scatter(x=df.index, y=df[col], name=col), secondary_y=tf)
        fig.update_yaxes(title_text=col, secondary_y=tf, showgrid=not tf)
    fig.update_layout(title=title)
    return fig



def read_rig_report(name):
    filename = fileloc % name
    ex = excel_scraper.read_excel_file(filename)
    tot_prod_list = []
    graphs = []
    for tab in sheets_to_graph:
        d = ex.parse(tab, index_col=0).iloc[:, :4]
        # reorganise the DF (as parameters don't work in parse)
        new_header = d.iloc[0]
        d = d.iloc[1:]
        d.columns = new_header
        title = d.index.name
        d.index.name = None
        tot_prod_list.append(d["Total production"].rename(title))
        fig = create_dualaxis_graph(d[['Production per rig', 'Rig count']], title=f"{title} Rig Count")
        graphs.append(fig)
        d['MoM'] = d['Total production'] - d['Total production'].shift(1)
        fig_mom = create_dualaxis_graph(d[['Total production', 'MoM']], title=f"{title} Production")
        graphs.append(fig_mom)
    total_prod = pd.concat(tot_prod_list, axis=1)
    fig = cpl.seas_line_plot(total_prod.sum(axis=1).astype(float),
                             title="Total Shale Production - 3 Main Plays",
                             precision_format='{0:,.0f}')
    graphs.append(fig)
    total_prod["YoY Changes"] = total_prod.sum(axis=1) - total_prod.sum(axis=1).shift(12)
    total_prod["YoY Average Changes"] = total_prod["YoY Changes"].rolling(window=12).mean()
    fig = cpl.bar_line_plot(total_prod[["YoY Changes", "YoY Average Changes"]],
                            linecol="YoY Average Changes",
                            title="US Shale Production")
    graphs.append(fig)
    return graphs


def read_duc_report(name):
    filename = fileloc % name
    ex = excel_scraper.read_excel_file(filename)
    d = ex.parse('Data', index_col=0)
    header = pd.MultiIndex.from_product([[x.strip() for x in list(d.iloc[1]) if str(x) != 'nan'],
                                         list(dict.fromkeys([x.strip() for x in list(d.iloc[2]) if str(x) != 'nan']))],
                                        names=['loc', 'drill_type'])
    d.dropna(axis=1, how='all', inplace=True)
    d.columns = header
    d = d.iloc[3:]
    locs = [x.replace(" Region", "") for x in sheets_to_graph]
    graphs = []
    for loc in locs:
        df = d.xs(loc, level='loc', axis=1)
        fig = cpl.seas_line_plot(df['DUC'].astype(float), shaded_range=[2017, 2020], title=f"{loc} DUCs")
        graphs.append(fig)
    return graphs


def get_jinja_dict():
    data = {'title': 'EIA Productivity Report', 'name': 'EIA Productivity Report'}
    data['release_date'] = read_release_date()
    data['rig_charts'] = read_rig_report(filenames[0])
    data['duc_charts'] = read_duc_report(filenames[1])

    return data


def generate_page(filename=None):
    return ju.render_html(data=get_jinja_dict(),
                          template='monthly_drilling_report.html',
                          filename=filename,
                          package_loader_name='oilanalytics.eia',
                          template_globals={'cu': cu})


def generate_email():
    compose_and_send_jinja_report('EIA Productivity Report',
                                  data=get_jinja_dict(),
                                  template='monthly_drilling_report_email.html',
                                  package_loader_name='oilanalytics.eia',
                                  template_globals={'cu': cu})

if __name__ == "__main__":

    proxy_support = urllib2.ProxyHandler({"http": "http://proxy-rwest-uk.energy.local:8080"})
    opener = urllib2.build_opener(proxy_support)
    urllib2.install_opener(opener)
    # graphs = read_rig_report(filenames[0])
    generate_page('test.html')
    # date = read_release_date()
    # generate_email()


import logging as logger
from datetime import datetime
from oilanalytics.utils import chartutils as cu
from commodplot import jinjautils as ju
from pyenergyaspects import eaapi

eur_diesel_ids = [11286, 11285, 11282, 11279, 11284, 11283, 11278, 11314, 11280, 11281]


def generate_page(
    file_path: str = None,
    title_url: str = None,
):
    data = {
        "name": "Europe Diesel Balance",
        "title": "Europe Diesel Balance",
    }

    date_from = datetime(2010, 1, 1)

    data["df"] = eaapi.read_timeseries(dataset_id=eur_diesel_ids, date_from=date_from)

    data["title_url"] = title_url

    logger.info(f"generated page to {file_path}")
    return ju.render_html(
        data=data,
        template="europe_diesel_balance.html",
        filename=file_path,
        package_loader_name="oilanalytics.energyaspects",
        template_globals={"cu": cu},
    )


if __name__ == "__main__":
    generate_page(file_path="europe_diesel_balance.html")



import glob
import os


def find_latest_file(fold):
    list_of_files = glob.glob(fold)
    list_of_files = [x for x in list_of_files if "~" not in x]
    latest_file = max(list_of_files, key=os.path.getctime)
    return latest_file



import os

import eikon as ek
from commodplot import commodplot as cpl
from commodplot import jinjautils as ju
from dotenv import load_dotenv

# Load environmental variables from '.env' file.
load_dotenv()

eikonkey = os.environ["EIKON_KEY"]
ek.set_app_key(eikonkey)

rics = {
    "CFSL-EU-EBBL": "Crude Stocks",
    "GLSL-EU-EBBL": "Gasoline Stocks",
    "DSTSL-EU-EBBL": "Distillate Stocks",
    "FOSL-EU-EBBL": "Fuel Stocks",
    "NAFSL-EU-EBBL": "Naphtha Stocks",
    "CRI1-EU-EBBL": "Refinery Intake",
    "CRI2-EU-EBBL": "Refinery Intake2",
    "GLRI-EU-EBBL": "Ref Output Gasoline",
    "DSTR-EU-EBBL": "Ref Output Distillate",
    "FORI-EU-EBBL": "Ref Output Fuel",
    "NAFRI-EU-EBBL": "Ref Output Naphtha",
    "REFO-EU-EBBL": "Ref Net Output",
}


def seas_chart(tarser, title):
    f = cpl.seas_line_plot(tarser, title=title, shaded_range=5)
    return f


def generate_euroil_stock_page(out_loc=None):
    if not out_loc:
        out_loc = "Euroil.html"

    data = {"name": "Euroil Stocks", "title": "Euroil Stocks"}
    start_date = "2012-01-01"
    df = ek.get_timeseries(
        list(rics.keys()), fields="Close", start_date=start_date
    ).rename(columns=rics)

    data["Crude Stocks"] = seas_chart(df["Crude Stocks"], title="Crude Stocks")
    data["Refinery Intake"] = seas_chart(df["Refinery Intake"], title="Refinery Intake")

    for product in ["Gasoline", "Distillate", "Naphtha", "Fuel"]:
        stocks = "%s Stocks" % product
        data[stocks] = seas_chart(df[stocks], title=stocks)
        supply = "Ref Output %s" % product
        supply_d = df[supply].dropna()
        data[supply] = seas_chart(supply_d, title=supply)

        # data['%s_supply'] = seas_chart(df[ric[1]].dropna(), title='%s Supply' % product)
        # yield_ = (df[ric[1]].dropna() / df['CRI1-EU-EBBL']).round(2)
        # data['%s_yield'] = seas_chart(pd.DataFrame(yield_), title='%s Supply' % product)

    ju.render_html(
        data,
        "euroilstock.html",
        filename=out_loc,
        package_loader_name="oilanalytics.euroilstock",
    )


if __name__ == "__main__":
    # balance_comparisons()
    generate_euroil_stock_page(r"M:\SitePages\Fundamentals\Euroilstock.aspx")
    # areachart()



import setuptools

setuptools.setup(
    name="oilanalytics",
    version="0.3.1",
    author="aeorxc",
    description="Utilities for oil analytics",
    url="https://github.com/aeorxc/oilanalytics",
    project_urls={
        "Source": "https://github.com/aeorxc/oilanalytics",
    },
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pandas",
        "requests",
        "commodutil",
        "commodplot",
        "python-dotenv",
        "cachetools",
        "beautifulsoup4",
        "eiapy",
    ],
    python_requires=">=3.8",
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    package_data={"oilanalytics": ["**/templates/*.html"]},
    include_package_data=True,
)



import pandas as pd
import pytest

from oilanalytics.balances import key_agency_comparisons


@pytest.mark.skip("TODO: Mock IEA ZIP file")
def test_balance_comparisons():
    res = key_agency_comparisons.balance_comparisons()
    assert isinstance(res, pd.DataFrame)



from commodplot import commodplot as cpl
from commodplot import jinjautils as ju

from oilanalytics.balances import balance_config, balance_utils
from oilanalytics.eia import steo
from oilanalytics.iea import omr
from oilanalytics.opec import momr


def balance_comparisons(additional_sources: dict = None):
    iea = omr.comparable_snd_table()
    iea = iea[iea.columns.intersection(balance_config.global_oil_cols)]
    opec = momr.comparable_snd_table()
    opec = opec[opec.columns.intersection(balance_config.global_oil_cols)]
    eia = steo.comparable_snd_table()
    eia = eia[eia.columns.intersection(balance_config.global_oil_cols)]

    res = {
        "iea": iea,
        "opec": opec,
        "eia": eia,
    }

    if additional_sources:
        for additional_source in additional_sources:
            res[additional_source] = additional_sources[additional_source]

    return res


def format_table(df):
    summary_table = cpl.table_plot(df, chtype="table", formatted_cols=["Change", "%"])
    format_dict = {"height": 30, "font": {"size": 10}}
    summary_table.update_traces({"cells": format_dict, "header": format_dict})
    return summary_table


def generate_comparisons_page(out_loc=None):
    if not out_loc:
        out_loc = "BalanceComparison.html"
    d = balance_comparisons()
    data = {"name": "Balance Comparisons", "title": "Balance Comparisons"}
    data["iea_table"] = balance_utils.quarter_summary_format(
        d["iea"][balance_config.global_oil_basic_summary_cols]
    )
    data["eia_table"] = balance_utils.quarter_summary_format(
        d["eia"]
    )  # [balance_config.global_oil_basic_summary_cols])
    data["opec_table"] = balance_utils.quarter_summary_format(
        d["opec"]
    )  # [balance_config.global_oil_basic_summary_cols])
    ju.render_html(
        data,
        "balance_comparisons.html",
        filename=out_loc,
        package_loader_name="oilanalytics.balances",
    )


if __name__ == "__main__":
    # balance_comparisons()
    generate_comparisons_page()
